from typing import Any

from snakypy.zshpower.prompt.sections.utils import Color, symbol_ssh


class Took:
    def __init__(self, config: dict, took: Any = 0):
        self.enable = config["took"]["enable"]
        self.symbol = symbol_ssh(config["took"]["symbol"], "")
        self.text = config["took"]["text"]
        self.color = (
            config["took"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.involved = config["took"]["involved"]
        self.show_greater_than = config["took"]["show_greater_than"]
        self.took = str(took)

    def format_took(self) -> str:
        if self.took and len(self.took) == 7:
            if self.took[0] == "0" and self.took[4] == "0":
                return f"{self.took[1:3]} {self.took[5:]}"
            elif self.took[0] == "0":
                return f"{self.took[1:3]} {self.took[4:]}"
            elif self.took[4] == "0":
                return f"{self.took[0:3]} {self.took[5:]}"
        else:
            if self.took[0] == "0":
                return self.took[1:3]
        return self.took

    def show(self):
        took_current = (
            f" {Color(self.color)}{self.symbol}{self.text} "
            f"{Color().NONE}{self.format_took()}"
        )
        if len(self.involved) == 2:
            took_current = (
                f" {self.involved[0]}{Color(self.color)}{self.symbol}{self.text} "
                f"{Color().NONE}{self.format_took()}{self.involved[1]}"
            )
        return took_current

    def __str__(self):
        if self.enable:
            if len(str(self.took)) == 7:
                return self.show()
            else:
                verify_seconds = f"{self.format_took()[:-1]}"
                if verify_seconds > str(self.show_greater_than):
                    return self.show()

        return ""



from getpass import getpass
from os.path import exists
from subprocess import PIPE, Popen

from snakypy.helpers import pick, printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.files import backup_file, create_file
from snakypy.helpers.os import remove_objects

from snakypy.zshpower import HOME, __info__
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.config.zshrc import zshrc_sample
from snakypy.zshpower.utils.catch import read_zshrc_omz
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.process import reload_zsh
from snakypy.zshpower.utils.shift import (
    change_theme_in_zshrc,
    rm_source_zshrc,
    uninstall_by_pip,
)


def finished() -> None:
    try:
        if exists(Base(HOME).sync_path) or exists(Base(HOME).cron_path):
            pass_ok = False

            message = """
                     You need to provide the superuser password to remove the process on ZSHPower on Cron.
                     If you do not want this step to be done, you can cancel with Ctrl + C.
                     """

            printer(message, foreground=FG().WARNING)

            while not pass_ok:
                sudo_password = getpass()

                command = f"""su -c 'rm -f {Base(HOME).sync_path}; rm -f {Base(HOME).cron_path}'"""
                p = Popen(
                    command,
                    stdin=PIPE,
                    stderr=PIPE,
                    stdout=PIPE,
                    universal_newlines=True,
                    shell=True,
                )
                communicate = p.communicate(sudo_password)

                if "failure" in communicate[1].split():
                    printer("Password incorrect.", foreground=FG().ERROR)
                else:
                    pass_ok = True

        reload_zsh()
        printer("Uninstall process finished.", foreground=FG().FINISH)
    except KeyboardInterrupt:
        reload_zsh()
        printer("Uninstall process finished.", foreground=FG().FINISH)


class UninstallCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self) -> None:
        checking_init(self.HOME, self.logfile)

        if not read_zshrc_omz(self.zsh_rc, self.logfile):
            title = f"Do you want to uninstall {__info__['name']}?"
            options = ["Yes", "No"]
            reply = pick(title, options, colorful=True, index=True)
            if reply is None or reply[0] == 1:
                printer("Whew! Thanks! :)", foreground=FG().GREEN)
                exit(0)
            remove_objects(objects=(self.init_file, self.data_root, self.cache_root))
            uninstall_by_pip(packages=(__info__["name"],))
            rm_source_zshrc(self.zsh_rc, self.logfile)
            finished()
        else:
            title = "What did you want to uninstall?"
            options = [
                f"{__info__['name']}",
                f"{__info__['name']} and Oh My ZSH",
                "Cancel",
            ]
            reply = pick(title, options, colorful=True, index=True)

            # Cancel
            if reply is None or reply[0] == 2:
                printer("Whew! Thanks! :)", foreground=FG().GREEN)
                exit(0)

            # Remove default
            remove_objects(
                objects=(
                    self.theme_file,
                    self.init_file,
                    self.data_root,
                    self.cache_root,
                )
            )
            uninstall_by_pip(packages=(__info__["name"],))
            change_theme_in_zshrc(self.zsh_rc, "robbyrussell", self.logfile)

            # ZSHPower and Oh My ZSH
            if reply[0] == 1:
                backup_file(self.zsh_rc, self.zsh_rc, date=True, extension=False)
                remove_objects(objects=(self.omz_root, self.zsh_rc, self.cache_root))
                create_file(zshrc_sample, self.zsh_rc, force=True)

            finished()



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Ruby(Version):
    def __init__(self):
        super(Ruby, self).__init__()
        self.files = ("Gemfile", "Rakefile")
        self.extensions = (".rb",)

    def get_version(
        self, config, reg_version, key="ruby", ext="rb-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="ruby", action=None) -> bool:
        version = run(
            "ruby --version 2>/dev/null",
            capture_output=True,
            shell=True,
            text=True,
        )

        if version.returncode != 127 and version.returncode != 1:
            version_format = (
                version.stdout.replace("\n", " ").split(" ")[1].split("p")[0]
            )
            return super().set(version_format, key, action)
        return False



from snakypy.helpers import printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.console import credence

from snakypy.zshpower import __info__


def billboard_manual() -> str:
    # This sign was generated by the "figlet" program (http://www.figlet.org/).
    # It also has a package called pyfiglet (https://pypi.org/project/pyfiglet/).
    text = """
                    ____              _
                   / ___| _ __   __ _| | ___   _ _ __  _   _
                   \___ \| '_ \ / _` | |/ / | | | '_ \| | | |
                    ___) | | | | (_| |   <| |_| | |_) | |_| |
                   |____/|_| |_|\__,_|_|\_\\\__, | .__/ \__, |
                                           |___/|_|    |___/
    """
    return text


class CreditsCommand:
    @staticmethod
    def run() -> None:
        print("\n")
        printer("Offered by:".center(50), foreground=FG().GREEN)
        printer(
            billboard_manual(),
            foreground=FG().YELLOW,
        )
        # billboard(
        #     __info__["organization_name"],
        #     justify="center",
        #     foreground=FG().YELLOW,
        # )
        printer("copyright (c) since 2020\n".center(100), foreground=FG().GREEN)
        credence(
            __info__["name"],
            __info__["version"],
            __info__["home_page"],
            __info__,
            foreground=FG().CYAN,
        )



import sqlite3
from textwrap import dedent
from typing import Any

from snakypy.helpers import FG, NONE

from snakypy.zshpower import HOME
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.database.sql import sql


class DAO(Base):
    def __init__(self):
        try:
            Base.__init__(self, HOME)
            self.conn = sqlite3.connect(self.database_path)
            self.get_cursor = self.conn.cursor()

        except sqlite3.Error:
            message = dedent(
                f"""\n\n{FG().RED}
                An error occurred while connecting to the database. Make sure that the SQLite database is turned on.
                One way to resolve it is by running the command "zshpower init [--omz]".{NONE}
            """
            )
            self.log.record(message, colorize=True)
            raise sqlite3.Error(message)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.commit()
        self.connection.close()

    @property
    def connection(self) -> sqlite3.Connection:
        return self.conn

    @property
    def cursor(self) -> sqlite3.Cursor:
        return self.get_cursor

    def commit(self):
        self.connection.commit()

    def execute(self, sql_, params=None):
        self.cursor.execute(sql_, params or ())

    def fetchall(self) -> list:
        return self.cursor.fetchall()

    def fetchone(self) -> Any:
        return self.cursor.fetchone()

    def query(self, sql_, params=None) -> list:
        self.cursor.execute(sql_, params or ())
        return self.fetchall()

    def query_one(self, sql_, params=None) -> Any:
        self.cursor.execute(sql_, params or ())
        return self.fetchone()

    def create_table(self, tbl_name) -> bool:
        try:
            self.execute(sql()[tbl_name])
            self.commit()
            self.connection.close()
            return True
        except (sqlite3.DatabaseError, sqlite3.DataError):
            return False

    def select_columns(self, /, columns=(), table=None) -> dict:
        sql_ = f"SELECT {','.join(columns)} FROM {table};"
        query = self.query(sql_)
        data = {key: value for (key, value) in query}
        self.connection.close()
        return data

    def select_where(self, table, value, where, select=()) -> list:
        sql_ = f"SELECT {','.join(select)} FROM {table} WHERE {where} = '{value}';"
        data = self.query(sql_)
        self.commit()
        self.connection.close()
        return data

    def insert(self, table, /, columns=(), values=()) -> bool:
        sql_ = f"INSERT INTO {table} {columns} VALUES {values};"
        try:
            self.execute(sql_)
            self.commit()
            self.connection.close()
            return True
        except Exception:
            raise Exception("Error insert database.")

    def update(self, table, set_, version, where, value) -> bool:
        sql_ = f"UPDATE {table} SET {set_} = '{version}' WHERE {where} = '{value}';"
        try:
            self.execute(sql_)
            self.commit()
            self.connection.close()
            return True
        except sqlite3.OperationalError as err:
            raise sqlite3.OperationalError(
                "Error writing to table. Table does not exist.", err
            )



from snakypy.helpers.catches import whoami

from snakypy.zshpower import HOME
from snakypy.zshpower.config.base import Base

cron_content: str = f"""
#!/bin/sh

PATH="~/.local/bin:/bin:/usr/local/bin:\\$PATH"
export PATH

# Hourly in minute 0
# 0 * * * * {whoami()} {Base(HOME).sync_path}

# At minute 0 past every 2nd hour
# 0 */2 * * * {whoami()} {Base(HOME).sync_path}

# Daily at 7:00 am
# 0 07 * * * {whoami()} {Base(HOME).sync_path}

# You can use the site "https://crontab.guru" and some examples at "https://crontab.guru/examples.html"
"""

sync_content: str = """
#!/bin/sh

PATH="~/.local/bin:/bin:/usr/local/bin:\\$PATH"
export PATH

zshpower sync

"""



from os import environ, getcwd
from os.path import exists, isdir, isfile, join
from sys import version_info

from snakypy.helpers.catches import is_tool
from snakypy.helpers.files import read_file

from snakypy.zshpower import HOME
from snakypy.zshpower.prompt.sections.utils import (
    Color,
    element_spacing,
    separator,
    symbol_ssh,
)
from snakypy.zshpower.utils.catch import verify_objects


def definitive_version(micro_version_enable, python_version, space_elem):
    if not micro_version_enable:
        return f"{'{0[0]}.{0[1]}'.format(python_version)}{space_elem}"
    return f"{'{0[0]}.{0[1]}.{0[2]}'.format(python_version)}{space_elem}"


class Python:
    def __init__(self, config):

        self.config = config
        self.enable = config["python"]["version"]["enable"]
        self.files = (
            "__pycache__",
            "manage.py",
            "setup.py",
            "__init__.py",
            ".python-version",
            "requirements.txt",
            "pyproject.toml",
        )
        self.folders = ("__pycache__",)
        self.extensions = (".py",)
        self.symbol = symbol_ssh(config["python"]["symbol"], "py-")
        self.color = (
            config["python"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.prefix_color = config["python"]["prefix"]["color"]
        self.prefix_text = element_spacing(config["python"]["prefix"]["text"])
        self.micro_version_enable = config["python"]["version"]["micro"]["enable"]

    def get_version(self, space_elem=" ") -> str:

        # Checking if you use Python through pyenv or the system.
        if isdir(join(HOME, ".pyenv")):

            pyenv_file_local = join(getcwd(), self.files[4])
            pyenv_file_global = join(HOME, ".pyenv/version")

            if exists(pyenv_file_local):
                if read_file(pyenv_file_local).strip() == "system":
                    python_version = (
                        f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(".")
                    )
                else:
                    python_version = read_file(pyenv_file_local).strip().split(".")
            elif exists(pyenv_file_global):
                if read_file(pyenv_file_global).strip() == "system":
                    python_version = (
                        f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(".")
                    )
                else:
                    python_version = read_file(pyenv_file_global).strip().split(".")
            else:
                python_version = f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(
                    "."
                )
        else:
            python_version = f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(".")

        return definitive_version(self.micro_version_enable, python_version, space_elem)

    def __str__(self):
        if self.enable:
            if is_tool("python") or is_tool("python3"):
                if (
                    verify_objects(
                        getcwd(),
                        files=self.files,
                        folders=self.folders,
                        extension=self.extensions,
                    )
                    or "VIRTUAL_ENV" in environ
                ):
                    prefix = (
                        f"{Color(self.prefix_color)}{self.prefix_text}{Color().NONE}"
                    )

                    return str(
                        f"{separator(self.config)}{prefix}"
                        f"{Color(self.color)}{self.symbol}"
                        f"{self.get_version()}{Color().NONE}"
                    )
        return ""


class Virtualenv:
    def __init__(self, config):
        self.config = config
        self.enable = config["python"]["virtualenv"]["enable"]
        self.hash_enable = config["python"]["virtualenv"]["poetry"]["hash"]["enable"]
        self.py_enable = config["python"]["virtualenv"]["poetry"]["py"]["enable"]
        self.symbol = symbol_ssh(config["python"]["virtualenv"]["symbol"], "")
        self.involved = config["python"]["virtualenv"]["involved"]
        self.color = (
            config["python"]["virtualenv"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.prefix_color = config["python"]["virtualenv"]["prefix"]["color"]
        self.prefix_text = element_spacing(
            config["python"]["virtualenv"]["prefix"]["text"]
        )
        self.name_enable = config["python"]["virtualenv"]["name"]["normal"]["enable"]
        self.name_text = config["python"]["virtualenv"]["name"]["text"]

    def get_virtualenv(self) -> str:
        if "VIRTUAL_ENV" in environ:
            venv_path = environ["VIRTUAL_ENV"]
            # The "venv_hash" option is only for virtual machines created with
            # poetry, where it generates a hash in the name of the virtual machine.
            if not self.hash_enable and isfile(join(getcwd(), "pyproject.toml")):
                venv_path = environ["VIRTUAL_ENV"].split("/")[-1]
                if "-" in venv_path:
                    return "-".join(venv_path.split("-")[:-2])
            # The "pg_version" option is only for virtual machines created with
            # poetry, where it generates the current version of Python in the name
            # of the virtual machine.
            if not self.py_enable and isfile(join(getcwd(), "pyproject.toml")):
                if "-" in venv_path:
                    return "-".join(venv_path.split("/")[-1].split("-")[:-1])

            return venv_path.split("/")[-1]
        return ""

    def __str__(self, space_elem=" "):
        involved_prefix = ""
        involved_suffix = ""

        if self.enable:
            if "VIRTUAL_ENV" in environ:
                prefix = (
                    f"{Color(self.prefix_color)}" f"{self.prefix_text}{Color().NONE}"
                )
                if len(self.involved) == 2:
                    involved_prefix = self.involved[0]
                    involved_suffix = self.involved[1]

                if self.name_enable:
                    ret = (
                        f"{separator(self.config)}{prefix}{Color(self.color)}"
                        f"{self.symbol}"
                        f"{involved_prefix}{self.get_virtualenv()}{involved_suffix}"
                        f"{space_elem}{Color().NONE}"
                    )
                else:
                    ret = (
                        f"{separator(self.config)}{prefix}{Color(self.color)}"
                        f"{self.symbol}"
                        f"{involved_prefix}{self.name_text}{involved_suffix}"
                        f"{space_elem}{Color().NONE}"
                    )
                return str(ret)
        return ""



from functools import wraps
from typing import Any


def assign_cli(arguments: dict, command: str) -> Any:
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            if arguments[command]:
                return func(*args, **kwargs)

        return wrapper

    return decorator



from os import environ, getcwd
from os.path import isdir, join

from snakypy.zshpower.prompt.sections.utils import (
    Color,
    element_spacing,
    git_status,
    separator,
    symbol_ssh,
)


class Git:
    def __init__(self, config, icon_space=" "):

        self.config = config
        # TODO: ZSHPower version 1.0.0 - In the future, the configuration
        #  file (config.toml) will no longer be mandatory in ZSHPower, as it
        #  is necessary to check if there are keys in the configuration file.
        try:
            self.symbol = symbol_ssh(config["git"]["symbol"], "git:")
        except KeyError:
            self.symbol = symbol_ssh("\uf418", "git:")
        self.enable = config["git"]["enable"]
        self.color_symbol = (
            config["git"]["color"]["symbol"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.branch_color = (
            config["git"]["branch"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.prefix_color = (
            config["git"]["prefix"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.prefix_text = element_spacing(config["git"]["prefix"]["text"])
        self.symbol_enable = config["git"]["status"]["symbols"]["enable"]
        self.icons = {
            "A": [
                f"{Color('green') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['added'], '')}"
                f"{Color().NONE}",
                f"{Color('green') if config['general']['color']['enable'] is True else Color('negative')}+"
                f"{icon_space}{Color().NONE}",
            ],
            # "AM": [
            #     f"{Color('white') if config['general']['color']['enable'] is True else Color('negative')}"
            #     f"{symbol_ssh(config_file['git']['status']['symbol']['changed'], '')}"
            #     f"{Color().NONE}",
            #     f"{Color('white') if config['general']['color']['enable'] is True else Color('negative')}#
            #     {icon_space}{Color().NONE}",
            # ],
            "M": [
                f"{Color('blue') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['modified'], '')}"
                f"{Color().NONE}",
                f"{Color('blue') if config['general']['color']['enable'] is True else Color('negative')}#"
                f"{icon_space}{Color().NONE}",
            ],
            "D": [
                f"{Color('red') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['deleted'], '')}"
                f"{Color().NONE}",
                f"{Color('red') if config['general']['color']['enable'] is True else Color('negative')}x"
                f"{icon_space}{Color().NONE}",
            ],
            "??": [
                f"{Color('yellow') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['untracked'], '')}"
                f"{Color().NONE}",
                f"{Color('yellow') if config['general']['color']['enable'] is True else Color('negative')}?"
                f"{icon_space}{Color().NONE}",
            ],
            "R": [
                f"{Color('magenta') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['renamed'], '')}"
                f"{Color().NONE}",
                f"{Color('magenta') if config['general']['color']['enable'] is True else Color('negative')}->"
                f"{icon_space}{Color().NONE}",
            ],
            "UU": [
                f"{Color('red') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['conflicts'], '')}"
                f"{Color().NONE}",
                f"{Color('red') if config['general']['color']['enable'] is True else Color('negative')}!="
                f"{icon_space}{Color().NONE}",
            ],
            "AH": [
                f"{Color('blue') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['ahead'], '')}"
                f"{Color().NONE}",
                f"{Color('blue') if config['general']['color']['enable'] is True else Color('negative')}^"
                f"{icon_space}{Color().NONE}",
            ],
            "BH": [
                f"{Color('magenta') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['behind'], '')}"
                f"{Color().NONE}",
                f"{Color('magenta') if config['general']['color']['enable'] is True else Color('negative')}_"
                f"{icon_space}{Color().NONE}",
            ],
            "DG": [
                f"{Color('yellow') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['diverged'], '')}"
                f"{Color().NONE}",
                f"{Color('yellow') if config['general']['color']['enable'] is True else Color('negative')}<->"
                f"{icon_space}{Color().NONE}",
            ],
            "C": [
                f"{Color('yellow') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['copied'], '')}"
                f"{Color().NONE}",
                f"{Color('yellow') if config['general']['color']['enable'] is True else Color('negative')}**"
                f"{icon_space}{Color().NONE}",
            ],
            "U": [
                f"{Color('magenta') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['unmerged'], '')}"
                f"{Color().NONE}",
                f"{Color('magenta') if config['general']['color']['enable'] is True else Color('negative')}="
                f"{icon_space}{Color().NONE}",
            ],
            "CL": [
                f"{Color('green') if config['general']['color']['enable'] is True else Color('negative')}"
                f"{symbol_ssh(config['git']['status']['symbol']['clean'], '')}"
                f"{Color().NONE}",
                f"{Color('green') if config['general']['color']['enable'] is True else Color('negative')}~"
                f"{icon_space}{Color().NONE}",
            ],
        }
        self.icons["UD"] = self.icons["UU"]

    def __str__(self):
        if isdir(join(getcwd(), ".git")) and self.enable:
            status_git = git_status(porcelain=True)
            status_git_text = git_status()
            branch_current = git_status(branch=True)
            branch_formated = (
                f"{Color(self.prefix_color)}"
                f"{self.prefix_text}{Color().NONE}"
                f"{Color(self.color_symbol)}"
                f"{self.symbol}{Color().NONE}"
                f"{Color(self.branch_color)}"
                f"{branch_current}"
                f"{Color().NONE}"
            )
            status_current = []
            if "??" in status_git:
                status_current.append("??")
            if "D" in status_git:
                status_current.append("D")
            if "R" in status_git:
                status_current.append("R")
            if "A" in status_git:
                status_current.append("A")
            # if "AM" in status_git:
            #     status_current.append("AM")
            if "M" in status_git:
                status_current.append("M")
            if "UU" in status_git:
                status_current.append("UU")
            if "U" in status_git:
                status_current.append("U")
            if "UD" in status_git:
                status_current.append("UD")
            if "C" in status_git:
                status_current.append("C")
            if "ahead" in status_git_text:
                status_current.append("AH")
            if "behind" in status_git_text:
                status_current.append("BH")
            if "diverged" in status_git_text:
                status_current.append("DG")
            if len(status_git) == 0:
                status_current.append("CL")

            status_icons = (
                self.icons[item][0]
                if self.symbol_enable
                else self.icons[item][1]
                if "SSH_CONNECTION" not in environ
                else self.icons[item][1]
                for item in status_current
            )

            # status = f'( {" ".join(sorted(status_icons)).strip()} )'
            # if len(status_current) == 1:
            status = "".join(sorted(status_icons))

            return f"{separator(self.config)}{branch_formated} {status}"
        return ""



from subprocess import call as subprocess_call

from snakypy.helpers import FG, printer
from snakypy.helpers.catches import shell, whoami

from snakypy.zshpower.utils.shift import log_base


def reload_zsh() -> None:
    subprocess_call("exec zsh", shell=True)


def change_shell(logfile) -> bool:
    if shell() != "zsh":
        try:
            subprocess_call(f"chsh -s $(which zsh) {whoami()}", shell=True)
            return True
        except KeyboardInterrupt:
            log_base(logfile).record(
                "Shell change canceled by user", colorize=True, level="info"
            )
            printer("Canceled by user.", foreground=FG().WARNING)
    return False



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Php(Version):
    def __init__(self):
        super(Php, self).__init__()
        self.files = ("composer.json",)
        self.extensions = (".php",)

    def get_version(
        self, config, reg_version, key="php", ext="php-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="php", action=None) -> bool:
        version = run(
            """php -v 2>&1 | grep "^PHP\\s*[0-9.]\\+" | awk '{print $2}'""",
            capture_output=True,
            shell=True,
            text=True,
        ).stdout

        version_format = version.replace("\n", "")

        if version_format:
            return super().set(version_format, key, action)

        return False



import sys
from concurrent.futures import ThreadPoolExecutor
from sqlite3 import OperationalError

from snakypy.helpers import FG
from snakypy.helpers.console import loading, printer

from snakypy.zshpower.commands.utils.handle import records
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.check import checking_init


class Sync(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self) -> None:
        try:
            checking_init(self.HOME, self.logfile)
            with ThreadPoolExecutor(max_workers=2) as executor:
                executor.submit(
                    loading,
                    set_time=0.100,
                    bar=False,
                    header="Synchronizing versions with database ...",
                    foreground=FG().QUESTION,
                )
                executor.submit(records, action="update")
            self.log.record("Database update.", colorize=True, level="info")
            printer("Done!", foreground=FG().FINISH)
        except KeyboardInterrupt:
            printer(
                "This operation cannot be canceled. Wait for the operation.",
                foreground=FG().WARNING,
            )
        except OperationalError:
            self.log.record(
                "The database does not exist or is corrupted.",
                colorize=True,
                level="error",
            )
            printer(
                "The database does not exist or is corrupted.",
                foreground=FG().ERROR,
            )
            sys.exit(1)



from os import getcwd
from subprocess import check_output
from typing import List, Union

from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.utils.catch import verify_objects


def symbol_ssh(symbol1, symbol2, spacing=" ") -> list:
    import os

    if symbol1 != "":
        symbol1 += spacing
    if "SSH_CONNECTION" in os.environ:
        symbol1 = symbol2
    return symbol1


def git_status(*, porcelain=False, branch=False) -> Union[str, List[str]]:
    porcelain_set = "--porcelain" if porcelain else ""
    branch_set = "--branch" if branch else ""

    if porcelain and not branch:
        status: Union[str, List[str]] = check_output(
            f"git status {porcelain_set}", shell=True, universal_newlines=True
        ).split()
    elif branch and not porcelain:
        data = check_output(
            f"git status {branch_set}", shell=True, universal_newlines=True
        ).split()
        if data[0] == "HEAD":
            status = f"{Color('red')}HEAD{Color().NONE}"
        else:
            status = data[2]
    else:
        status = check_output(
            f"git status {porcelain_set} {branch_set}",
            shell=True,
            universal_newlines=True,
        ).split()
    return status


def separator(config, spacing=" ") -> str:
    sep = config["general"]["separator"]["element"]
    sep += spacing if sep != "" else sep
    sep_color = config["general"]["separator"]["color"]
    separator_style = f"{Color(sep_color)}{sep}{Color().NONE}"
    return separator_style


def element_spacing(element, spacing=" "):
    if element != "":
        element += spacing
    return element


class Color:
    """Color application compatible only ZSH."""

    NONE = "%f"

    def __init__(self, set_color=""):
        self.color = f"%F{{{set_color}}}"
        if set_color == "negative":
            self.color = ""

    def __str__(self):
        return self.color


class Version(DAO):
    def __init__(self):
        DAO.__init__(self)
        self.extensions = ()
        self.files = ()
        self.folders = ()

    def get(self, config, reg_version: dict, key="", ext="", space_elem="") -> str:
        enable = config[key]["version"]["enable"]
        symbol = symbol_ssh(config[key]["symbol"], ext)
        color = (
            config[key]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        prefix_color = config[key]["prefix"]["color"]
        prefix_text = element_spacing(config[key]["prefix"]["text"])
        micro_version_enable = config[key]["version"]["micro"]["enable"]

        if enable:
            if reg_version[key] and verify_objects(
                getcwd(),
                files=self.files,
                folders=self.folders,
                extension=self.extensions,
            ):
                prefix = f"{Color(prefix_color)}{prefix_text}{Color().NONE}"

                if micro_version_enable:
                    version_format = f"{'{0[0]}.{0[1]}.{0[2]}'.format(reg_version[key].split('.'))}{space_elem}"
                else:
                    version_format = f"{'{0[0]}.{0[1]}'.format(reg_version[key].split('.'))}{space_elem}"

                return str(
                    (
                        f"{separator(config)}{prefix}"
                        f"{Color(color)}{symbol}"
                        f"{version_format}{Color().NONE}"
                    )
                )
        return ""

    def set(self, version, key="", action=None) -> bool:
        if action:
            # Conditions
            if action == "insert":
                query = DAO().select_where(
                    self.tbl_main, key, "name", select=("version",)
                )
                if not query:
                    DAO().insert(
                        self.tbl_main,
                        columns=("name", "version"),
                        values=(key, version),
                    )

            elif action == "update":
                DAO().update(self.tbl_main, "version", version, "name", key)
            return True
        return False



from os import environ, geteuid
from pathlib import Path
from subprocess import run

from .utils import Color, element_spacing, symbol_ssh


def get_pwd() -> str:
    # import os
    # from os import getenv, system
    # from os import environ, getcwd, getcwdb, getenvb
    # from pathlib import Path
    # from os import getcwd

    # NOTES: https://stackoverflow.com/questions/123958/how-to-get-set-logical-directory-path-in-python
    # return os.popen('pwd').read().strip('\n')
    # return getenv('PWD')
    # return getcwd()
    return run("pwd", capture_output=True, shell=True, text=True).stdout.replace(
        "\n", ""
    )


def shorten_path(file_path, length) -> Path:
    return Path(*Path(file_path).parts[-length:])


class Directory:
    def __init__(self, config):
        self.username_enable = config["username"]["enable"]
        self.hostname_enable = config["hostname"]["enable"]
        self.truncate_value = config["directory"]["truncation_length"]
        self.symbol = symbol_ssh(config["directory"]["symbol"], "")
        self.color = (
            config["directory"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.prefix_color = config["directory"]["prefix"]["color"]
        self.prefix_text = element_spacing(config["directory"]["prefix"]["text"])

    def __str__(self, prefix="", space_elem=" "):
        if (
            self.username_enable
            or geteuid() == 0
            or self.hostname_enable
            or "SSH_CONNECTION" in environ
        ):
            prefix = f"{Color(self.prefix_color)}" f"{self.prefix_text}{Color().NONE}"

        if int(self.truncate_value) < 0:
            self.truncate_value = 0
        if int(self.truncate_value) > 4:
            self.truncate_value = 4

        # Old "abspath_link()"
        dir_truncate = str(shorten_path(get_pwd(), self.truncate_value))

        if dir_truncate.split("/")[-1:] == str(Path.home()).split("/")[-1:]:
            dir_truncate = "~"
        return (
            f"{prefix}{Color(self.color)}{self.symbol}"
            f"{dir_truncate}{space_elem}{Color().NONE}"
        )



import concurrent.futures
from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class NodeJs(Version):
    def __init__(self):
        super(NodeJs, self).__init__()
        self.files = ("package.json",)
        self.folders = ("node_modules",)

    def get_version(
        self, config, reg_version, key="nodejs", ext="node-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="nodejs", action=None) -> bool:
        version = run("node -v 2>/dev/null", capture_output=True, shell=True, text=True)

        if version.returncode != 127 and version.returncode != 1:
            version_format = version.stdout.replace("\n", "").split("v")[1]
            return super().set(version_format, key, action)

        return False


def _nodejs(config, key):
    with concurrent.futures.ThreadPoolExecutor() as executor:
        future = executor.submit(NodeJs().get_version, config, key)
        return_value = future.result()
        return return_value



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Perl(Version):
    def __init__(self):
        super(Perl, self).__init__()
        self.extensions = (".pl",)

    def get_version(
        self, config, reg_version, key="perl", ext="pl-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="perl", action=None) -> bool:
        version = run(
            # Model
            # perl -Mstrict -wall -e "print join('.', map {ord} split('', \$^V));"
            # perl -v | awk '/This/ {print $4}' | sed -e 's/v//'
            "perl -version | awk '/version/' 2>&1",
            capture_output=True,
            text=True,
            shell=True,
        ).stdout

        if version.replace("\n", ""):
            version_format = (
                version.split()[8].replace("v", "").replace("(", "").replace(")", "")
            )
            return super().set(version_format, key, action)

        return False



from contextlib import suppress
from os import getcwd
from os.path import exists, isfile, join
from subprocess import run

from snakypy.helpers.files import read_json

from snakypy.zshpower.utils.catch import verify_objects

from .utils import Color, element_spacing, separator, symbol_ssh


class Base:
    def __init__(self, config):
        self.config = config
        self.files = ()
        self.folders = ()
        self.extensions = ()
        self.symbol = symbol_ssh(config["package"]["symbol"], "pkg-")
        self.enable = config["package"]["enable"]
        self.color = (
            config["package"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.prefix_color = config["package"]["prefix"]["color"]
        self.prefix_text = element_spacing(config["package"]["prefix"]["text"])

    def get_version(self):
        return ""

    def __str__(self):
        if self.enable:
            package_version = self.get_version()
            if package_version and verify_objects(
                getcwd(),
                files=self.files,
                folders=self.folders,
                extension=self.extensions,
            ):
                prefix = (
                    f"{Color(self.prefix_color)}" f"{self.prefix_text}{Color().NONE}"
                )
                return (
                    f"{separator(self.config)}{prefix}"
                    f"{Color(self.color)}"
                    f"{self.symbol}{package_version}{Color().NONE}"
                )
        return ""


class Python(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.files = (
            "pyproject.toml",
            "manage.py",
            "requirements.txt",
            "setup.py",
            "__init__.py",
        )

    def get_version(self, space_elem=" ") -> str:
        if self.enable:
            python_package_version = run(
                f"""< {self.files[0]} grep "^version = *" | cut -d'"' -f2 | cut -d"'" -f2""",
                capture_output=True,
                shell=True,
                text=True,
            ).stdout

            python_package_version = python_package_version.replace("\n", "")

            if python_package_version:
                return f"{python_package_version}{space_elem}"
        return ""

    def __str__(self):
        return super().__str__()


class NodeJS(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.files = ("package.json",)
        self.folders = ("node_modules",)

    def get_version(self, space_elem=" "):
        if self.enable and isfile(join(getcwd(), self.files[0])):
            with suppress(Exception):
                parsed = read_json(join(getcwd(), self.files[0]))
                if "version" in parsed:
                    return f"{parsed['version']}{space_elem}"
        return ""

    def __str__(self):
        return Base.__str__(self)


class Rust(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.files = ("Cargo.toml",)

    def get_version(self, space_elem=" ") -> str:
        if self.enable:
            python_package_version = run(
                f"""< {self.files[0]} grep "^version := *" | cut -d'"' -f2 | cut -d"'" -f2""",
                capture_output=True,
                shell=True,
                text=True,
            ).stdout

            python_package_version = python_package_version.replace("\n", "")

            if python_package_version:
                return f"{python_package_version}{space_elem}"
        return ""

    def __str__(self):
        return super().__str__()


# TODO: Future development
class CMake:
    # CMakeLists.txt
    pass


class Scala(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.files = ("build.sbt",)

    def get_version(self, space_elem=" ") -> str:
        if self.enable:
            scala_package_version = run(
                f"""< {self.files[0]} grep "^version := *" | cut -d'"' -f2 | cut -d"'" -f2""",
                capture_output=True,
                shell=True,
                text=True,
            ).stdout

            try:
                scala_package_version = scala_package_version.replace("\n", "").split()[
                    -1
                ]
            except IndexError:
                scala_package_version = scala_package_version.replace("\n", "")

            if scala_package_version:
                return f"{scala_package_version}{space_elem}"
        return ""


class Crystal(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.files = ("shard.yml",)

    def get_version(self, space_elem=" ") -> str:
        if self.enable:
            crystal_package_version = run(
                f"""< {self.files[0]} grep "^version: *" | cut -d'"' -f2 | cut -d"'" -f2""",
                capture_output=True,
                shell=True,
                text=True,
            ).stdout

            try:
                crystal_package_version = crystal_package_version.replace(
                    "\n", ""
                ).split()[1]
            except IndexError:
                crystal_package_version = crystal_package_version.replace("\n", "")

            if crystal_package_version:
                return f"{crystal_package_version}{space_elem}"
        return ""


class Helm(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.files = ("Chart.yaml",)

    def get_version(self, space_elem=" ") -> str:
        if self.enable:
            helm_package_version = run(
                f"""< {self.files[0]} grep "^version: *" | cut -d'"' -f2 | cut -d"'" -f2""",
                capture_output=True,
                shell=True,
                text=True,
            ).stdout

            try:
                helm_package_version = helm_package_version.replace("\n", "").split()[1]
            except IndexError:
                helm_package_version = helm_package_version.replace("\n", "")

            if helm_package_version:
                return f"{helm_package_version}{space_elem}"
        return ""


class Package:
    def __init__(self, config):
        self.config = config

    def __str__(self):
        if exists(join(getcwd(), Python(self.config).files[0])):
            return str(Python(self.config))
        elif exists(join(getcwd(), Rust(self.config).files[0])):
            return str(Rust(self.config))
        elif exists(join(getcwd(), NodeJS(self.config).files[0])):
            return str(NodeJS(self.config))
        elif exists(join(getcwd(), Scala(self.config).files[0])):
            return str(Scala(self.config))
        elif exists(join(getcwd(), Crystal(self.config).files[0])):
            return str(Crystal(self.config))
        elif exists(join(getcwd(), Helm(self.config).files[0])):
            return str(Helm(self.config))
        return ""



from concurrent.futures import ThreadPoolExecutor

from snakypy.helpers import printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.console import loading

from snakypy.zshpower.commands.utils.handle import records
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.config.config import content as config_content
from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.process import reload_zsh
from snakypy.zshpower.utils.shift import create_config


class ResetCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self, arguments) -> None:
        checking_init(self.HOME, self.logfile)
        if arguments["--config"]:
            create_config(config_content, self.config_file, force=True)
            printer("Reset process finished.", foreground=FG().FINISH)
            self.log.record("Config files reset", colorize=True, level="info")
            reload_zsh()
        elif arguments["--db"]:
            DAO().create_table(self.tbl_main)
            with ThreadPoolExecutor(max_workers=2) as executor:
                executor.submit(
                    loading,
                    set_time=0.140,
                    bar=False,
                    header="ZSHPower Restoring the database ...",
                    foreground=FG().QUESTION,
                )
                executor.submit(records, action="insert")
            printer("Done!", foreground=FG().FINISH)
            self.log.record("Database reset", colorize=True, level="info")



from concurrent.futures import ThreadPoolExecutor
from contextlib import suppress
from sqlite3 import OperationalError
from sys import argv as sys_argv
from sys import stdout
from typing import Any

from snakypy.helpers import FG, printer
from snakypy.helpers.ansi import NONE
from snakypy.helpers.decorators import only_linux
from snakypy.helpers.files import read_file
from snakypy.helpers.path import create as create_path
from tomlkit import parse as toml_parse
from tomlkit.exceptions import NonExistentKey, UnexpectedCharError

from snakypy.zshpower import __info__
from snakypy.zshpower.config.config import content as config_content
from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.prompt.sections.cmake import CMake
from snakypy.zshpower.prompt.sections.command import Command
from snakypy.zshpower.prompt.sections.crystal import Crystal
from snakypy.zshpower.prompt.sections.dart import Dart
from snakypy.zshpower.prompt.sections.deno import Deno
from snakypy.zshpower.prompt.sections.directory import Directory
from snakypy.zshpower.prompt.sections.docker import Docker
from snakypy.zshpower.prompt.sections.dotnet import Dotnet
from snakypy.zshpower.prompt.sections.elixir import Elixir
from snakypy.zshpower.prompt.sections.erlang import Erlang
from snakypy.zshpower.prompt.sections.git import Git
from snakypy.zshpower.prompt.sections.golang import Golang
from snakypy.zshpower.prompt.sections.gulp import Gulp
from snakypy.zshpower.prompt.sections.helm import Helm
from snakypy.zshpower.prompt.sections.hostname import Hostname
from snakypy.zshpower.prompt.sections.java import Java
from snakypy.zshpower.prompt.sections.julia import Julia
from snakypy.zshpower.prompt.sections.jump_line import JumpLine
from snakypy.zshpower.prompt.sections.kotlin import Kotlin
from snakypy.zshpower.prompt.sections.nim import Nim
from snakypy.zshpower.prompt.sections.nodejs import NodeJs
from snakypy.zshpower.prompt.sections.ocaml import Ocaml
from snakypy.zshpower.prompt.sections.package import Package
from snakypy.zshpower.prompt.sections.perl import Perl
from snakypy.zshpower.prompt.sections.php import Php
from snakypy.zshpower.prompt.sections.python import Python, Virtualenv
from snakypy.zshpower.prompt.sections.ruby import Ruby
from snakypy.zshpower.prompt.sections.rust import Rust
from snakypy.zshpower.prompt.sections.scala import Scala
from snakypy.zshpower.prompt.sections.timer import Timer
from snakypy.zshpower.prompt.sections.took import Took
from snakypy.zshpower.prompt.sections.username import Username
from snakypy.zshpower.prompt.sections.vagrant import Vagrant
from snakypy.zshpower.prompt.sections.zig import Zig
from snakypy.zshpower.utils.shift import create_config

# ## Test timer ## #
# from snakypy.helpers.decorators import runtime


class Draw(DAO):
    def __init__(self):
        DAO.__init__(self)
        self.config = self.get_config()
        self.register = self.get_register()

    def get_config(self) -> dict:
        try:
            parsed = dict(toml_parse(read_file(self.config_file)))
            return parsed

        except (FileNotFoundError, NonExistentKey):
            create_path(self.zshpower_home)
            create_config(config_content, self.config_file)
            parsed = dict(toml_parse(read_file(self.config_file)))
            self.log.record(
                "Configuration files does not exist, however it was created.",
                colorize=True,
                level="error",
            )
            return parsed

    def get_register(self):
        try:
            data = self.select_columns(
                columns=("name", "version"),
                table=self.tbl_main,
            )
            return data
        except (KeyError, OperationalError):
            self.log.record(
                "Corrupted database. Firing guidance message.",
                colorize=True,
                level="error",
            )
            printer(
                f'{__info__["name"]} Error: Database corrupted. Run command: '
                f'"zshpower reset --db" to restore.\n>> ',
                foreground=FG().ERROR,
            )

    def version(self, instance, key) -> str:
        with ThreadPoolExecutor() as executor:
            if key in self.register:
                future = executor.submit(
                    instance().get_version, self.config, self.register
                )
                return_value = future.result()
                return return_value
            return ""

    @staticmethod
    def get_keys(dict_: dict, key: Any) -> str:
        return dict_[key]

    # @runtime
    def prompt(self, took: Any = 0) -> str:
        try:
            with suppress(KeyboardInterrupt):
                jump_line = JumpLine(self.config)
                username = Username(self.config)
                hostname = Hostname(self.config)
                directory = Directory(self.config)
                dinamic_section = {
                    "virtualenv": Virtualenv(self.config),
                    "python": Python(self.config),
                    "package": Package(self.config),
                    "nodejs": self.version(NodeJs, "nodejs"),
                    "rust": self.version(Rust, "rust"),
                    "golang": self.version(Golang, "golang"),
                    "ruby": self.version(Ruby, "ruby"),
                    "dart": self.version(Dart, "dart"),
                    "php": self.version(Php, "php"),
                    "java": self.version(Java, "java"),
                    "julia": self.version(Julia, "julia"),
                    "dotnet": self.version(Dotnet, "dotnet"),
                    "elixir": self.version(Elixir, "elixir"),
                    "scala": self.version(Scala, "scala"),
                    "perl": self.version(Perl, "perl"),
                    "cmake": self.version(CMake, "cmake"),
                    "crystal": self.version(Crystal, "crystal"),
                    "deno": self.version(Deno, "deno"),
                    "erlang": self.version(Erlang, "erlang"),
                    "helm": self.version(Helm, "helm"),
                    "kotlin": self.version(Kotlin, "kotlin"),
                    "nim": self.version(Nim, "nim"),
                    "ocaml": self.version(Ocaml, "ocaml"),
                    "vagrant": self.version(Vagrant, "vagrant"),
                    "zig": self.version(Zig, "zig"),
                    "gulp": self.version(Gulp, "gulp"),
                    "docker": self.version(Docker, "docker"),
                    "git": Git(self.config),
                }

                cmd = Command(self.config)
                took_ = Took(self.config, took)
                static_section = f"{jump_line}{username}{hostname}{directory}"

                # Using ThreadPoolExecutor, not Generators
                with ThreadPoolExecutor() as executor:
                    ordered_section = []
                    for elem in self.config["general"]["position"]:
                        for item in dinamic_section.keys():
                            if item == elem:
                                future = executor.submit(
                                    self.get_keys, dinamic_section, item
                                )
                                ordered_section.append(future.result())

                sections = "{}{}{}" + "{}" * len(dinamic_section)
                return sections.format(static_section, *ordered_section, took_, cmd)

        except (NonExistentKey, UnexpectedCharError, ValueError):
            self.log.record(
                "Key error in the configuration file.", colorize=True, level="error"
            )
            print(
                f"{FG().ERROR}{__info__['name']} Error: Key error in "
                f"the configuration file.\n> {NONE}"
            )
            raise
        except KeyError:
            self.log.record(
                "Corrupted database. Firing guidance message.",
                colorize=True,
                level="error",
            )
            raise KeyError(
                f"{FG().ERROR}{__info__['name']} Error: Database records are missing "
                f"or corrupted. Run the command to correct: \"{__info__['executable']} reset --db\".\n>> "
            )
        return ""

    def rprompt(self) -> str:
        try:
            timer = Timer(self.config)
            return f"{timer}"
        except NonExistentKey:
            self.log.record(
                "Corrupted database. Firing guidance message.",
                colorize=True,
                level="error",
            )
            return (
                f"{FG().ERROR}>>> {__info__['name']} Error: Key error in "
                f"the configuration file.\n > "
            )


@only_linux
def main() -> None:
    if len(sys_argv) < 2:
        raise TypeError("missing 1 required positional argument")
    if len(sys_argv) == 3 and sys_argv[1] == "prompt" and sys_argv[2]:
        stdout.write(Draw().prompt(took=sys_argv[2]))
    elif len(sys_argv) == 2 and sys_argv[1] == "prompt":
        stdout.write(Draw().prompt())
    elif len(sys_argv) == 2 and sys_argv[1] == "rprompt":
        stdout.write(Draw().rprompt())



from os import environ as os_environ
from os import geteuid

from snakypy.helpers.catches import whoami

from .utils import Color, symbol_ssh


class Username:
    def __init__(self, config):
        self.symbol = symbol_ssh(config["username"]["symbol"], "")
        self.enable = config["username"]["enable"]
        self.color = config["username"]["color"]
        if geteuid() == 0:
            self.color = "red"

    def __str__(self, space_elem=" "):
        if self.enable or "SSH_CONNECTION" in os_environ or geteuid() == 0:
            return (
                f"{Color(self.color)}{self.symbol}{whoami()}{space_elem}{Color().NONE}"
            )
        return ""



from os import environ
from pydoc import pager as pydoc_pager
from shutil import which as shutil_which
from subprocess import call as subprocess_call

from snakypy.helpers.files import read_file
from tomlkit import parse as toml_parse

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.check import checking_init


def editor_run(editor, config) -> bool:
    if shutil_which(editor):
        get_editor = environ.get("EDITOR", editor)
        with open(config) as f:
            subprocess_call([get_editor, f.name])
            return True
    return False


class ConfigCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self, arguments) -> bool:
        checking_init(self.HOME, self.logfile)

        if arguments["--open"]:
            try:
                read_conf = read_file(self.config_file)
                parsed = dict(toml_parse(read_conf))
                editor_conf = parsed["general"]["config"]["editor"]
                if editor_conf:
                    editor_run(editor_conf, self.config_file)
                else:
                    editors = ("vim", "nano", "emacs", "micro")
                    for edt in editors:
                        editor_run(edt, self.config_file)
                self.log.record(
                    f"Open/Changed Configuration Files ({self.config_file}).",
                    colorize=True,
                    level="info",
                )
            except FileNotFoundError:
                raise FileNotFoundError("File not found.")
        elif arguments["--view"]:
            read_config = read_file(self.config_file)
            pydoc_pager(read_config)
            return True
        return False



from .utils import Color, symbol_ssh


class Command:
    def __init__(self, config):
        self.config = config
        self.symbol = symbol_ssh(config["command"]["symbol"], "> ")
        self.error_symbol = symbol_ssh(config["command"]["error"]["symbol"], "x ")
        self.error_color = config["command"]["error"]["color"]
        self.color = (
            config["command"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.new_line = config["command"]["new_line"]["enable"]

    def __str__(self, jump_line="\n"):
        if not self.new_line:
            jump_line = ""
        return (
            f"{jump_line}{Color(self.color)}"
            f"%(?.{self.symbol}."
            f"{Color(self.error_color)}"
            f"{self.error_symbol})"
            f"{Color().NONE}"
        )



from subprocess import run
from typing import Union

from snakypy.zshpower.prompt.sections.utils import Version


class Docker(Version):
    def __init__(self):
        super(Docker, self).__init__()
        self.files = ("Dockerfile", "docker-compose.yml")

    def get_version(
        self, config, reg_version, key="docker", ext="dkr-", space_elem=" "
    ) -> Union[str, bool]:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="docker", action=None) -> bool:
        version = run(
            "docker version",
            capture_output=True,
            text=True,
            shell=True,
        )

        if version.returncode != 127 and version.returncode != 1:
            version_format = (
                version.stdout.split("Version")[1]
                .strip()
                .split("\n")[0]
                .replace(":", "")
                .strip()
            )
            return super().set(version_format, key, action)

        return False



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class CMake(Version):
    def __init__(self):
        super(CMake, self).__init__()
        self.files = ("CMakeLists.txt", "CMakeCache.txt")

    def get_version(self, config, version, key="cmake", ext="cm-", space_elem=" "):
        return super().get(config, version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="cmake", action=None):
        version = run(
            "cmake --version 2>&1", capture_output=True, shell=True, text=True
        )
        if not version.returncode == 0:
            return False

        version = version.stdout.split()[2]

        return super().set(version, key, action)



from snakypy.helpers import printer
from snakypy.helpers.ansi import FG

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.catch import read_zshrc_omz
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.process import reload_zsh
from snakypy.zshpower.utils.shift import change_theme_in_zshrc


class DeactivateCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self, arguments, *, theme_name="robbyrussell") -> bool:

        checking_init(self.HOME, self.logfile)

        if read_zshrc_omz(self.zsh_rc, self.logfile):
            if not read_zshrc_omz(self.zsh_rc, self.logfile)[0] == "zshpower":
                printer("Already disabled. Nothing to do.", foreground=FG().GREEN)
                exit(0)
            if not arguments["--theme"]:
                change_theme_in_zshrc(self.zsh_rc, theme_name, self.logfile)
            else:
                change_theme_in_zshrc(self.zsh_rc, arguments["--theme"], self.logfile)
            printer("Deactivation process finish.", foreground=FG().FINISH)
            reload_zsh()
            return True
        printer(
            "You are not using Oh My ZSH to run this command.",
            foreground=FG().WARNING,
        )
        return False



from pydoc import pager

from snakypy.helpers import printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.files import read_file

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.shift import log_base


class LogsCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self, arguments) -> None:
        checking_init(self.HOME, self.logfile)
        if arguments["--view"]:
            read_logs = read_file(self.logfile)
            pager(read_logs)
        elif arguments["--clean"]:
            log_base(self.logfile, force=True)
            printer("Logs have been cleaned up.", foreground=FG().FINISH)



from concurrent.futures import ThreadPoolExecutor
from os.path import join

from snakypy.helpers import printer
from snakypy.helpers.ansi import FG, NONE
from snakypy.helpers.catches import tools_requirements
from snakypy.helpers.console import loading
from snakypy.helpers.files import create_file
from snakypy.helpers.path import create as snakypy_path_create

from snakypy.zshpower import __info__
from snakypy.zshpower.commands.utils.handle import records
from snakypy.zshpower.config.apply import content as set_zshpower_content
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.config.config import content as config_content
from snakypy.zshpower.config.cron import cron_content, sync_content
from snakypy.zshpower.config.zshrc import content as zshrc_content
from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.utils.catch import get_line_source
from snakypy.zshpower.utils.process import change_shell, reload_zsh
from snakypy.zshpower.utils.shift import (
    add_plugins_zshrc,
    change_theme_in_zshrc,
    create_config,
    create_zshrc,
    cron_task,
    install_fonts,
    omz_install,
    omz_install_plugins,
    remove_versions_garbage,
)

instruction_not_omz = f"""{FG().YELLOW}
********************** WARNING **********************
Add the following code to the {FG().MAGENTA}$HOME/.zshrc{NONE} {FG().YELLOW}file:

CODE: {FG().CYAN}source $HOME/.zshpower/{__info__["version"]}/init.sh {NONE}
{FG().YELLOW}*****************************************************{NONE}
"""


class InitCommand(Base):
    def __init__(self, home):
        Base.__init__(self, home)

    def run(self, arguments, *, reload=False, message=False) -> None:
        tools_requirements("bash", "zsh", "vim", "git", "cut", "grep", "whoami", "pwd")
        printer("Please wait ... assigning settings ...", foreground=FG().WARNING)
        snakypy_path_create(self.data_root, self.cache_root)
        create_config(config_content, self.config_file)
        create_file(set_zshpower_content, self.init_file, force=True)
        # Create table if not exists
        DAO().create_table(self.tbl_main)
        # Insert registers
        try:
            with ThreadPoolExecutor(max_workers=2) as executor:
                executor.submit(
                    loading,
                    set_time=0.140,
                    bar=False,
                    header="ZSHPower is creating the database. Wait a moment ...",
                    foreground=FG().QUESTION,
                )
                executor.submit(records, action="insert")
        except KeyboardInterrupt:
            printer(
                "This operation cannot be canceled. Wait for the operation.",
                foreground=FG().WARNING,
            )

        if arguments["--omz"]:
            omz_install(self.omz_root, self.logfile)
            omz_install_plugins(self.omz_root, self.plugins, self.logfile)
            create_zshrc(zshrc_content, self.zsh_rc, self.logfile)
            change_theme_in_zshrc(self.zsh_rc, f"{__info__['pkg_name']}", self.logfile)
            add_plugins_zshrc(self.zsh_rc, self.logfile)
            create_file(set_zshpower_content, self.theme_file, force=True)

        install_fonts(self.HOME, self.logfile)
        change_shell(self.logfile)
        remove_versions_garbage(join(self.HOME, f".{__info__['pkg_name']}"))

        try:
            cron_task(sync_content, self.sync_path, cron_content, self.cron_path)
            printer("Done!", foreground=FG().FINISH) if message else None

            if not arguments["--omz"] and not get_line_source(
                self.zsh_rc, self.logfile
            ):
                printer(instruction_not_omz, foreground=FG().YELLOW)

            if reload:
                reload_zsh()
        except KeyboardInterrupt:
            printer("Canceled by user", foreground=FG().WARNING)
            printer("Done!", foreground=FG().FINISH) if message else None

            if not arguments["--omz"] and not get_line_source(
                self.zsh_rc, self.logfile
            ):
                printer(instruction_not_omz, foreground=FG().YELLOW)

            if reload:
                reload_zsh()
        self.log.record("Initial settings applied", colorize=True, level="info")



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Dotnet(Version):
    def __init__(self):
        super(Dotnet, self).__init__()
        self.files = ("project.json", "global.json", "paket.dependencies")
        self.extensions = (".csproj", ".fsproj", ".xproj", ".sln")

    def get_version(
        self, config, reg_version, key="dotnet", ext=".net-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="dotnet", action=None) -> bool:

        version = run(
            "dotnet --version 2>/dev/null",
            capture_output=True,
            shell=True,
            text=True,
        ).stdout

        if not version.replace("\n", ""):
            return False
        version_format = version.replace("\n", "")
        return super().set(version_format, key, action)



"""CLI - Command Line Interface"""
try:
    from snakypy.helpers.decorators import only_linux, silent_errors
except KeyboardInterrupt:
    pass
from snakypy.zshpower import HOME
from snakypy.zshpower.commands.activate import ActivateCommand
from snakypy.zshpower.commands.config import ConfigCommand
from snakypy.zshpower.commands.credits import CreditsCommand
from snakypy.zshpower.commands.deactivate import DeactivateCommand
from snakypy.zshpower.commands.init import InitCommand
from snakypy.zshpower.commands.logs import LogsCommand
from snakypy.zshpower.commands.reset import ResetCommand
from snakypy.zshpower.commands.sync import Sync
from snakypy.zshpower.commands.uninstall import UninstallCommand
from snakypy.zshpower.utils.catch import arguments
from snakypy.zshpower.utils.decorators import assign_cli

# Get arguments Docopt
args: dict = arguments()


@assign_cli(args, "init")
def run_init() -> None:
    InitCommand(HOME).run(args, reload=True, message=True)


@assign_cli(args, "config")
def run_config() -> None:
    ConfigCommand(HOME).run(args)


@assign_cli(args, "activate")
def run_activate() -> None:
    ActivateCommand(HOME).run()


@assign_cli(args, "deactivate")
def run_deactivate() -> None:
    DeactivateCommand(HOME).run(args)


@assign_cli(args, "reset")
def run_reset() -> None:
    ResetCommand(HOME).run(args)


@assign_cli(args, "uninstall")
def run_uninstall() -> None:
    UninstallCommand(HOME).run()


@assign_cli(args, "sync")
def run_sync() -> None:
    Sync(HOME).run()


@assign_cli(args, "logs")
def run_logs() -> None:
    LogsCommand(HOME).run(args)


@assign_cli(args, "--credits")
def run_credits() -> None:
    CreditsCommand().run()


@silent_errors
@only_linux
def main() -> None:
    run_init()
    run_config()
    run_activate()
    run_deactivate()
    run_reset()
    run_uninstall()
    run_sync()
    run_logs()
    run_credits()



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Scala(Version):
    def __init__(self):
        super(Scala, self).__init__()
        self.extensions = (".scala", ".sc")
        self.files = ("build.sbt",)

    def get_version(
        self, config, reg_version, key="scala", ext="sc-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="scala", action=None) -> bool:
        version = run(
            "scala -version 2>&1",
            capture_output=True,
            text=True,
            shell=True,
        )

        if version.returncode != 127 and version.returncode != 1:
            version_format = version.stdout.split("-")[0].split()[4]
            return super().set(version_format, key, action)

        return False



from subprocess import run
from typing import Union

from snakypy.zshpower.prompt.sections.utils import Version


class Crystal(Version):
    def __init__(self):
        super(Crystal, self).__init__()
        self.extensions = (".cr",)
        self.files = ("shard.yml",)

    def get_version(
        self, config, reg_version, key="crystal", ext="cr-", space_elem=" "
    ) -> Union[str, bool]:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="crystal", action=None) -> bool:
        version = run(
            "crystal version 2>&1", capture_output=True, shell=True, text=True
        )

        if version.returncode != 127 and version.returncode != 1:
            version_format = version.stdout.split()[1]
            return super().set(version_format, key, action)

        return False



from os.path import exists, join

from snakypy.zshpower import __info__


def checking_init(home, logfile) -> bool:
    """Function that ends commands that depend on the created repository, but
    the repository was not created."""

    if not exists(join(home, f".{__info__['pkg_name']}")):
        from snakypy.zshpower.utils.shift import log_base

        log_base(logfile).record(
            f'Command "{__info__["pkg_name"]} init" has not been started.' "Aborted",
            colorize=True,
        )
        raise FileNotFoundError(
            f'Command "{__info__["pkg_name"]} init" has not been started.' "Aborted",
        )
    return True



import os
from contextlib import suppress
from os.path import exists, isdir
from re import M as re_m
from re import search as re_search

from docopt import docopt
from snakypy.helpers.ansi import FG, NONE

from snakypy.zshpower import __info__
from snakypy.zshpower.config import menu


def read_zshrc(zshrc, logfile) -> str:
    try:
        with open(zshrc) as f:
            return f.read()
    except FileNotFoundError as err:
        from snakypy.zshpower.utils.shift import log_base

        log_base(logfile).record("File not found.", colorize=True)
        raise FileNotFoundError("File not found.", err)


def arguments(argv=None) -> dict:
    formatted_version = (
        f"{__info__['name']} version: {FG().CYAN}{__info__['version']}{NONE}"
    )
    data = docopt(menu.options, argv=argv, version=formatted_version)
    return data


def read_zshrc_omz(zshrc, logfile) -> tuple:
    """ """
    current_zshrc = read_zshrc(zshrc, logfile)
    m = re_search(r"ZSH_THEME=\".*", current_zshrc)
    if m is not None:
        var_zsh_theme = m.group(0)
        lst = var_zsh_theme.split("=")
        theme_name = [s.strip('"') for s in lst][1]
        return theme_name, var_zsh_theme
    return ()


def plugins_current_zshrc(zshrc, logfile) -> list:
    current_zshrc = read_zshrc(zshrc, logfile)
    m = re_search(r"^plugins=\(.*", current_zshrc, flags=re_m)
    if m is not None:
        get = m.group(0)
        lst = get.split("=")
        current = [i.strip('"').replace("(", "").replace(")", "") for i in lst][1]
        return current.split()
    return []


def get_line_source(zshrc, logfile) -> str:
    current_zshrc = read_zshrc(zshrc, logfile)
    m = re_search(r"source \$HOME/.zshpower", current_zshrc)
    if m is not None:
        return m.group(0)
    return ""


def verify_objects(directory, /, files=(), folders=(), extension=()) -> bool:
    with suppress(PermissionError):
        for file in os.listdir(directory):
            if folders:
                for i in folders:
                    if isdir(i):
                        return True
            if extension:
                for i in extension:
                    obj = os.path.join(directory, i)
                    if not isdir(obj) and file.endswith(i):
                        return True
        if files:
            for i in files:
                if exists(os.path.join(directory, i)):
                    return True
    return False



class JumpLine:
    def __init__(self, config):
        self.enable = config["general"]["jump_line"]["enable"]

    def __str__(self):
        if not self.enable:
            return ""
        return "\n"



from os import getcwd
from os.path import isfile, join
from subprocess import run

from snakypy.helpers.files import read_json

from snakypy.zshpower.prompt.sections.utils import (
    Color,
    Version,
    element_spacing,
    separator,
    symbol_ssh,
)
from snakypy.zshpower.utils.catch import verify_objects


class Gulp(Version):
    def __init__(self):
        super(Gulp, self).__init__()
        self.files = ("gulpfile.js", "gulpfile.babel.js")
        self.folders = ("node_modules",)

    def get_version(
        self, config, reg_version, key="gulp", ext="gulp-", space_elem=" "
    ) -> str:
        version_local = "node_modules/gulp/package.json"
        enable = config[key]["version"]["enable"]
        symbol = symbol_ssh(config[key]["symbol"], ext)
        color = config[key]["color"]
        prefix_color = config[key]["prefix"]["color"]
        prefix_text = element_spacing(config[key]["prefix"]["text"])
        micro_version_enable = config[key]["version"]["micro"]["enable"]

        if enable and verify_objects(
            getcwd(),
            files=self.files,
            folders=self.folders,
            extension=self.extensions,
        ):
            prefix = f"{Color(prefix_color)}{prefix_text}{Color().NONE}"
            if isfile(join(getcwd(), version_local)):
                parsed = read_json(join(getcwd(), version_local))
                if "version" in parsed:
                    version_ = parsed["version"]

                    if micro_version_enable:
                        version_format = f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_.split('.'))}{space_elem}"
                    else:
                        version_format = (
                            f"{'{0[0]}.{0[1]}'.format(version_.split('.'))}{space_elem}"
                        )

                    return str(
                        (
                            f"{separator(config)}{prefix}"
                            f"{Color(color)}{symbol}"
                            f"Local {version_format}{Color().NONE}"
                        )
                    )
            else:
                return super().get(
                    config, reg_version, key=key, ext=ext, space_elem=space_elem
                )
        return ""

    def set_version(self, key="gulp", action=None) -> bool:
        version = run("gulp --version", capture_output=True, shell=True, text=True)

        if version.returncode != 127 and version.returncode != 1:
            version_format = version.stdout.split()[2]
            return super().set(f"CLI {version_format}", key, action)

        return False



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Elixir(Version):
    def __init__(self):
        super(Elixir, self).__init__()
        self.files = ("mix.exs",)
        self.extensions = (".ex",)

    def get_version(
        self, config, reg_version, key="elixir", ext="ex-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="elixir", action=None) -> bool:

        version = run(
            "elixir -v 2>/dev/null | grep 'Elixir' | cut -d ' ' -f2",
            capture_output=True,
            shell=True,
            text=True,
        ).stdout

        if version.replace("\n", ""):
            version_format = version.replace("\n", "")
            return super().set(version_format, key, action)
        return False



from concurrent.futures import ThreadPoolExecutor

from snakypy.zshpower.prompt.sections.cmake import CMake
from snakypy.zshpower.prompt.sections.crystal import Crystal
from snakypy.zshpower.prompt.sections.dart import Dart
from snakypy.zshpower.prompt.sections.deno import Deno
from snakypy.zshpower.prompt.sections.docker import Docker
from snakypy.zshpower.prompt.sections.dotnet import Dotnet
from snakypy.zshpower.prompt.sections.elixir import Elixir
from snakypy.zshpower.prompt.sections.erlang import Erlang
from snakypy.zshpower.prompt.sections.golang import Golang
from snakypy.zshpower.prompt.sections.gulp import Gulp
from snakypy.zshpower.prompt.sections.helm import Helm
from snakypy.zshpower.prompt.sections.java import Java
from snakypy.zshpower.prompt.sections.julia import Julia
from snakypy.zshpower.prompt.sections.kotlin import Kotlin
from snakypy.zshpower.prompt.sections.nim import Nim
from snakypy.zshpower.prompt.sections.nodejs import NodeJs
from snakypy.zshpower.prompt.sections.ocaml import Ocaml
from snakypy.zshpower.prompt.sections.perl import Perl
from snakypy.zshpower.prompt.sections.php import Php
from snakypy.zshpower.prompt.sections.ruby import Ruby
from snakypy.zshpower.prompt.sections.rust import Rust
from snakypy.zshpower.prompt.sections.scala import Scala
from snakypy.zshpower.prompt.sections.vagrant import Vagrant
from snakypy.zshpower.prompt.sections.zig import Zig


def records(action) -> None:
    with ThreadPoolExecutor(max_workers=10) as executor:
        executor.submit(Dart().set_version, action=action)
        executor.submit(Docker().set_version, action=action)
        executor.submit(Dotnet().set_version, action=action)
        executor.submit(Elixir().set_version, action=action)
        executor.submit(Golang().set_version, action=action)
        executor.submit(Gulp().set_version, action=action)
        executor.submit(Java().set_version, action=action)
        executor.submit(Julia().set_version, action=action)
        executor.submit(NodeJs().set_version, action=action)
        executor.submit(Php().set_version, action=action)
        executor.submit(Ruby().set_version, action=action)
        executor.submit(Rust().set_version, action=action)
        executor.submit(Scala().set_version, action=action)
        executor.submit(Perl().set_version, action=action)
        executor.submit(CMake().set_version, action=action)
        executor.submit(Crystal().set_version, action=action)
        executor.submit(Deno().set_version, action=action)
        executor.submit(Erlang().set_version, action=action)
        executor.submit(Helm().set_version, action=action)
        executor.submit(Kotlin().set_version, action=action)
        executor.submit(Nim().set_version, action=action)
        executor.submit(Ocaml().set_version, action=action)
        executor.submit(Vagrant().set_version, action=action)
        executor.submit(Zig().set_version, action=action)



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Dart(Version):
    def __init__(self):
        super(Dart, self).__init__()
        self.extensions = (".dart",)
        self.files = (
            "pubspec.yaml",
            "config.src.yaml",
            "analysis_options.yaml",
        )

    def get_version(
        self, config, reg_version, key="dart", ext="dt-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="dart", action=None) -> bool:
        version = run("dart --version 2>&1", capture_output=True, shell=True, text=True)

        if not version.returncode == 0:
            return False
        version_format = version.stdout.replace("\n", "").split(" ")[3]
        return super().set(version_format, key, action)



from time import strftime

from snakypy.zshpower.prompt.sections.utils import Color, symbol_ssh


class Timer:
    def __init__(self, config):
        self.enable = config["timer"]["enable"]
        self.symbol = symbol_ssh(config["timer"]["symbol"], "T:")
        self.color = (
            config["timer"]["color"]
            if config["general"]["color"]["enable"] is True
            else "negative"
        )
        self.seconds_enable = config["timer"]["seconds"]["enable"]

    def __str__(self):
        if self.enable:
            get_timer = str(strftime("%H:%M"))
            if self.seconds_enable:
                get_timer = str(strftime("%H:%M:%S"))
            timer = f"{Color(self.color)}{self.symbol}" f"{get_timer}{Color().NONE}"
            return timer.strip()
        return ""



from os.path import join

from snakypy.helpers.path import create as create_path

from snakypy.zshpower import __info__
from snakypy.zshpower.database.sql import sql
from snakypy.zshpower.utils.shift import log_base


class Base:
    def __init__(self, home):
        self.HOME = home
        self.zshpower_home = join(
            self.HOME, f".{__info__['pkg_name']}", __info__["version"]
        )
        self.tbl_main = [item for item in sql().keys()][0]
        self.config_file = join(self.zshpower_home, "config.toml")
        self.data_root = join(self.zshpower_home, ".data")
        self.cache_root = join(self.zshpower_home, ".cache")
        self.logfile = join(self.zshpower_home, ".cache", "zshpower.log")
        self.database_path = join(self.data_root, "db.sqlite3")
        self.sync_path = f"/usr/local/bin/{__info__['pkg_name']}_sync.sh"
        self.cron_path = f"/etc/cron.d/{__info__['pkg_name']}_task.sh"
        self.init_file = join(self.zshpower_home, "init.sh")
        self.zsh_rc = join(self.HOME, ".zshrc")
        self.omz_root = join(self.HOME, ".oh-my-zsh")
        self.themes_folder = join(self.omz_root, "custom/themes")
        self.theme_file = join(self.themes_folder, f"{__info__['pkg_name']}.zsh-theme")
        self.plugins = ("zsh-syntax-highlighting", "zsh-autosuggestions")
        create_path(self.cache_root)
        self.log = log_base(self.logfile)



from getpass import getpass
from os import remove, walk
from os.path import exists, isdir, isfile, join
from re import M as re_m
from re import sub as re_sub
from shutil import rmtree, which
from subprocess import PIPE, Popen, check_output
from sys import platform
from textwrap import dedent
from zipfile import ZipFile

from snakypy.helpers import FG, printer
from snakypy.helpers.files import backup_file, create_file
from snakypy.helpers.logging import Log
from snakypy.helpers.path import create as snakypy_path_create
from snakypy.helpers.subprocess import command
from tomlkit import dumps as toml_dumps
from tomlkit import parse as toml_parse

from snakypy.zshpower import __info__
from snakypy.zshpower.utils.catch import (
    plugins_current_zshrc,
    read_zshrc,
    read_zshrc_omz,
)


def create_config(content, file_path, *, force=False) -> bool:
    if not exists(file_path) or force:
        parsed_toml = toml_parse(content)
        write_toml = toml_dumps(parsed_toml)
        create_file(write_toml, file_path, force=force)
        return True
    return False


def log_base(filename, force=False) -> Log:
    content = dedent(
        f"""
        *******************************
         {__info__['name']} Logs - version {__info__['version']}
        *******************************

        | Level | Date | Message |

    """
    )
    if not exists(filename):
        create_file(content, filename, force=force)
    else:
        if force:
            create_file(content, filename, force=force)
    return Log(filename=filename)


def create_zshrc(content, zshrc, logfile) -> bool:
    if exists(zshrc):
        if not read_zshrc_omz(zshrc, logfile):
            backup_file(zshrc, zshrc, date=True, extension=False)
            create_file(content, zshrc, force=True)
            return True
    elif not exists(zshrc):
        create_file(content, zshrc)
        return True
    return False


def cron_task(sync_context, sync_path, cron_context, cron_path):

    if not exists(sync_path) or not exists(cron_path):
        pass_ok = False

        message = """
                At this point, you need to INFORM the root password to create the Crontab task.
                If you do not want this configuration to be made, you can cancel with Ctrl + C.
                """

        printer(message, foreground=FG().WARNING)

        while not pass_ok:
            sudo_password = getpass()

            command_ = f"""su -c 'echo "{sync_context}" > {sync_path}; chmod a+x {sync_path};
            echo "{cron_context}" > {cron_path};'
            """
            p = Popen(
                command_,
                stdin=PIPE,
                stderr=PIPE,
                stdout=PIPE,
                universal_newlines=True,
                shell=True,
            )
            communicate = p.communicate(sudo_password)

            if "failure" in communicate[1].split():
                printer("Password incorrect.", foreground=FG().ERROR)
            else:
                pass_ok = True


def change_theme_in_zshrc(zshrc, theme_name, logfile) -> bool:
    if read_zshrc_omz(zshrc, logfile):
        current_zshrc = read_zshrc(zshrc, logfile)
        current_theme = read_zshrc_omz(zshrc, logfile)[1]
        new_theme = f'ZSH_THEME="{theme_name}"'
        new_zsh_rc = re_sub(rf"{current_theme}", new_theme, current_zshrc, flags=re_m)
        create_file(new_zsh_rc, zshrc, force=True)
        return True
    return False


def omz_install(omz_root, logfile):
    omz_github = "https://github.com/ohmyzsh/ohmyzsh.git"
    cmd_line = f"git clone {omz_github} {omz_root}"
    try:
        if not exists(omz_root):
            printer("Install Oh My ZSH...", foreground=FG().QUESTION)
            command(cmd_line, verbose=True)
            printer(
                "Oh My ZSH installation process finished.",
                foreground=FG().FINISH,
            )

    except Exception:
        log_base(logfile).record("Error downloading Oh My ZSH. Aborted!", colorize=True)
        raise Exception("Error downloading Oh My ZSH. Aborted!")


def omz_install_plugins(omz_root, plugins, logfile):
    try:
        url_master = "https://github.com/zsh-users"
        for plugin in plugins:
            path = join(omz_root, f"custom/plugins/{plugin}")
            clone = f"git clone {url_master}/{plugin}.git {path}"
            if not isdir(path):
                printer(f"Install plugins {plugin}...", foreground=FG().QUESTION)
                command(clone, verbose=True)
                printer(f"Plugin {plugin} task finished!", foreground=FG().FINISH)
    except Exception:
        log_base(logfile).record(
            "There was an error installing the plugin", colorize=True
        )
        raise Exception("There was an error installing the plugin")


def install_fonts(home, logfile, *, force=False) -> bool:
    url = "https://github.com/snakypy/assets"
    base_url = "raw/main/zshpower/fonts/terminal/fonts.zip"
    font_name = "DejaVu Sans Mono Nerd Font"
    fonts_dir = join(home, ".fonts")
    snakypy_path_create(fonts_dir)
    curl_output = join(home, "zshpower__font.zip")

    if platform.startswith("linux"):
        try:
            if (
                not isfile(join(fonts_dir, "DejaVu Sans Mono Nerd Font Complete.ttf"))
                or force
            ):
                printer(
                    f'Please wait, downloading the "{font_name}" font and'
                    "installing...",
                    foreground=FG().QUESTION,
                )
                cmd_line = f"curl -L {join(url, base_url)} -o {curl_output}"
                command(cmd_line, verbose=True)

                with ZipFile(curl_output, "r") as zip_ref:
                    zip_ref.extractall(fonts_dir)
                    remove(curl_output)
                    printer("Done!", foreground=FG().FINISH)
                return True
            return False
        except Exception as err:
            log_base(logfile).record(
                f'Error downloading font "{font_name}"', colorize=True
            )
            raise Exception(f'Error downloading font "{font_name}"', err)
    return False


def add_plugins_zshrc(zshrc, logfile):
    plugins = (
        "python",
        "pip",
        "pep8",
        "autopep8",
        "zsh-syntax-highlighting",
        "zsh-autosuggestions",
    )
    current = plugins_current_zshrc(zshrc, logfile)

    new_plugins = [plugin for plugin in plugins if plugin not in current]

    if len(new_plugins) > 0:
        current_zshrc = read_zshrc(zshrc, logfile)
        plugins = f'plugins=({" ".join(current)} {" ".join(new_plugins)})'
        new_zsh_rc = re_sub(r"^plugins=\(.*", plugins, current_zshrc, flags=re_m)
        create_file(new_zsh_rc, zshrc, force=True)
        return new_zsh_rc
    return ""


def rm_source_zshrc(zshrc, logfile):
    current_zshrc = read_zshrc(zshrc, logfile)
    line_rm = f"source\\ \\$HOME/.zshpower/{__info__['version']}/init.sh"
    new_zshrc = re_sub(rf"{line_rm}", "", current_zshrc, flags=re_m)
    create_file(new_zshrc, zshrc, force=True)


def uninstall_by_pip(*, packages=()) -> tuple:
    if which("pip") is not None:
        for pkg in packages:
            check_output(
                f"pip uninstall {pkg} -y",
                shell=True,
                universal_newlines=True,
            )
    return packages


def remove_versions_garbage(path) -> None:
    folders = []
    for root, dirs, files in walk(path):
        for item in dirs:
            folders.append(item)

    for folder in folders:
        if join(path, folder) != join(path, __info__["version"]):
            rmtree(join(path, folder), ignore_errors=True)



from subprocess import run

from snakypy.zshpower.prompt.sections.utils import Version


class Golang(Version):
    def __init__(self):
        super(Golang, self).__init__()
        self.extensions = (".go",)
        self.files = ("go.mod", "glide.yaml")
        self.folders = ("Godeps",)

    def get_version(
        self, config, reg_version, key="golang", ext="go-", space_elem=" "
    ) -> str:
        return super().get(config, reg_version, key=key, ext=ext, space_elem=space_elem)

    def set_version(self, key="golang", action=None) -> bool:
        version = run("go version", capture_output=True, shell=True, text=True)

        if not version.stderr.replace("\n", ""):
            version_format = version.stdout.replace("go", "").split(" ")[2]
            return super().set(version_format, key, action)

        return False
